# -*- coding: utf-8 -*-

# MeninGIT -- Web-based wrapper of git
# By: Romain SOUFFLET <rsoufflet@easter-eggs.com>
#
# Copyright (C) 2011 Easter-eggs
# http://wiki.infos-pratiques.org/wiki/MeninGIT
#
# This file is part of meninGIT.
#
# MeninGIT is free software; you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# MeninGIT is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


"""Root controllers"""


import os

import webob.exc
from time import time

from dulwich.errors import NotGitRepository
from dulwich.repo import Repo
from dulwich.objects import Commit, parse_timezone

from .. import templates
from ..config import conf, urls
from ..lib import contexts, wsgihelpers
from ..lib.git import create_tree, parse_tree, parse_repo


@wsgihelpers.wsgify
def commit(req):
    ctx = contexts.Ctx(req)

    time_zone = parse_timezone('-0200')
    commit_message = req.params.get('commit-message') or None
    site = req.params.get('site-name') or None
    repo_name = req.params.get('repo-name') or None
    if repo_name == None:
        return webob.exc.HTTPFound(location = urls.url(ctx, 'index'))
    else:
        repo_path = conf['repos'][site][repo_name.encode('utf-8')]

    if type(time_zone) != int:
        time_zone = time_zone[0]

    # Get the last Commit
    try:
        repo = Repo(repo_path)
        repo_head_sha1 = repo.head()
    except KeyError:
        raise wsgihelpers.bad_request(ctx, message = "%s ne contient pas de références 'HEAD'" % repo_path)
    except:
        raise wsgihelpers.bad_request(ctx, message = "%s n'est pas un repository Git" % repo_path)
    repo_last_commit = repo[repo_head_sha1]
    if not repo_last_commit:
        raise webob.exc.HTTPNotFound('No commit')

    # on vide l'index, pour le reremplir plus tard
    repo_index = repo.open_index()
    repo_index.clear()
    commit_tree = create_tree(repo)

    # Define the new Commit
    commit = Commit()
    commit.tree = commit_tree.id
    commit.parents = [repo.head()]
    commit.author = commit.committer = "MeninGIT <support@easter-eggs.com>"
    commit.commit_time = commit.author_time = int(time())
    commit.commit_timezone  = commit.author_timezone = time_zone
    commit.message = commit_message

    # Commit it
    object_store = repo.object_store
    object_store.add_object(commit)

    # Refresh Index with good informations
    #       ctime   = (int(info.st_ctime), 0)
    #       mtime   = (int(info.st_mtime), 0)
    #       dev     = int(info.st_dev)
    #       ino     = info.st_ino
    #       mode    = info.st_mode
    #       uid     = info.st_uid
    #       gid     = info.st_gid
    #       size    = info.st_size
    #       sha     = blob[1].id
    #       flags   = 0
    repo_index_entries = dict()
    for blob in parse_tree(repo, commit_tree, ''):
        blob_path = "%s/%s" % (repo_path, blob[0])
        if os.path.isfile(blob_path):
            blob_info = os.stat(blob_path)
            mode = blob_info.st_mode
        if os.path.islink(blob_path):
            blob_info = os.lstat(blob_path)
            mode = blob_info.st_mode
            mode = mode - (mode & int("000111111111", 2))

        repo_index_entries[blob[0]] = (
            (int(blob_info.st_ctime), 0),
            (int(blob_info.st_mtime), 0),
            int(blob_info.st_dev),
            blob_info.st_ino,
            mode,
            blob_info.st_uid,
            blob_info.st_gid,
            blob_info.st_size,
            blob[1].id,
            0
            )

    repo_index.update(repo_index_entries)
    repo_index.write()

    # Refresh refs
    repo.refs['refs/heads/master'] = commit.id
    repo.refs['HEAD'] = commit.id

    return webob.exc.HTTPFound(location = urls.url(ctx, 'diff', repo_name = repo_name, site = site))


@wsgihelpers.wsgify
def commit_index(req):
    ctx = contexts.Ctx(req)

    time_zone = parse_timezone('-0200')
    commit_message = req.params.get('commit-message') or None
    site = req.params.get('site-name') or None
    repo_name = req.params.get('repo-name') or None
    if repo_name == None:
        return webob.exc.HTTPFound(location = urls.url(ctx, 'index'))
    else:
        repo_path = conf['repos'][site][repo_name.encode('utf-8')]

    if type(time_zone) != int:
        time_zone = time_zone[0]

    # Get the last Commit
    try:
        repo = Repo(repo_path)
    except KeyError:
        raise wsgihelpers.bad_request(ctx, message = "%s ne contient pas de références 'HEAD'" % repo_path)
    except:
        raise wsgihelpers.bad_request(ctx, message = "%s n'est pas un repository Git" % repo_path)
    repo_last_commit = repo[repo.head()]
    if not repo_last_commit:
        raise webob.exc.HTTPNotFound('No commit')

    # On crée le Tree à partir de l'index
    repo_index = repo.open_index()
    tree_sha1 = repo_index.commit(repo.object_store)

    # Define the new Commit
    commit = Commit()
    commit.tree = tree_sha1
    commit.parents = [repo.head()]
    commit.author = commit.committer = "MeninGIT <support@easter-eggs.com>"
    commit.commit_time = commit.author_time = int(time())
    commit.commit_timezone  = commit.author_timezone = time_zone
    commit.message = commit_message

    # Commit it
    repo.object_store.add_object(commit)

    # Refresh refs
    repo.refs['refs/heads/master'] = commit.id
    repo.refs['HEAD'] = commit.id

    return webob.exc.HTTPFound(location = urls.url(ctx, 'diff_from_index', repo_name = repo_name, site = site))


@wsgihelpers.wsgify
def diff(req):
    ctx = contexts.Ctx(req)

    site =  req.params.get('site') or None
    repo_name =  req.params.get('repo-name') or None

    if repo_name == None or site == None:
        return webob.exc.HTTPFound(location = urls.url(ctx, 'index'))
    else:
        repo_path = conf['repos'][site][repo_name]

    # Get the list of Commit
    try:
        repo = Repo(repo_path)
        repo_head_sha1 = repo.head()
    except KeyError:
        raise wsgihelpers.bad_request(ctx, message = "%s ne contient pas de références 'HEAD'" % repo_path)
    except:
        raise wsgihelpers.bad_request(ctx, message = "%s n'est pas un repository Git" % repo_path)
    repo_commits = repo.revision_history(repo_head_sha1)
    if not repo_commits:
        raise webob.exc.HTTPNotFound('No commit')

    # Get the list of actual File
    files_path = list()
    for path in os.walk(repo_path):
        for directory in path[1]:
            if os.path.islink("%s/%s" % (path[0], directory)):
                relative_file_name = "%s/%s" % (path[0].replace(repo_path, ''), directory)
                relative_file_name = relative_file_name[1:]
                files_path.append(relative_file_name)
        if path[0].rfind(".git") == -1:
            for file in path[2]:
                relative_file_name = "%s/%s" % (path[0].replace(repo_path, ''), file)
                relative_file_name = relative_file_name[1:]
                files_path.append(relative_file_name)
    files_path.sort()

    # Get the list of blob in last commit
    repo_blobs = parse_repo(repo_path)

    # File modified
    blob_list=list()
    path_list=list()
    modif_list=list()
    for blob in repo_blobs:
        blob_list.append(blob[0])
    for path in files_path:
        path_list.append(path)

    # list file wich has been created or deleted
    for blob in blob_list:
        if blob not in path_list:
            modif_list.append((blob, "delete"))
    for path in path_list:
        if path not in blob_list:
            modif_list.append((path, "create"))

    return templates.render(ctx, '/diff.mako', repo_path = repo_path, repo_name = repo_name, repo_commits =
            repo_commits, files_path = files_path, repo_blobs = repo_blobs, modif_list = modif_list, site = site)


@wsgihelpers.wsgify
def diff_from_index(req):
    ctx = contexts.Ctx(req)

    site =  req.params.get('site') or None
    repo_name =  req.params.get('repo-name') or None
    commit_sha1 = req.params.get('commit_sha1') or None

    if repo_name is None or site is None:
        message = u"Les données reçu sont incomplete"
        return templates.render(ctx, '/erreur.mako', message = message)

    repo_path = conf['repos'][site][repo_name]
    try:
        repo = Repo(repo_path)
    except NotGitRepository:
        raise wsgihelpers.bad_request(ctx, message = "%s n'est pas un repository GIT" % repo_path)
    except KeyError:
        raise wsgihelpers.bad_request(ctx, message = "%s ne contient pas de références 'HEAD'" % repo_path)
    except:
        raise wsgihelpers.bad_request(ctx, message = "%s n'est pas un repository Git" % repo_path)
    repo_commits = repo.revision_history(repo.head())
    if not repo_commits:
        raise webob.exc.HTTPNotFound('No commit')

    if commit_sha1 is None:
        commit_sha1 = repo.head()

    # get the Tree of the Commit and his parent
    commit = repo[commit_sha1]
    tree_sha1 = commit.tree

    repo_index = repo.open_index()
    changes = list(repo_index.changes_from_tree(repo.object_store, tree_sha1))

    modif_list = []
    for change in changes:
        if change[0][0] is None:
            modif_list.append((change[0][1], "create"))
        elif change[0][1] is None:
            modif_list.append((change[0][0], "delete"))

    return templates.render(ctx, '/diff.mako', modif_list = modif_list, repo_path = repo_path, repo_name = repo_name,
            site = site, changes = changes, repo_commits = repo_commits)


@wsgihelpers.wsgify
def erreur(req):
    ctx = contexts.Ctx(req)

    message = req.params.get('message') or None
    return templates.render(ctx, '/erreur.mako', message = message)


@wsgihelpers.wsgify
def index(req):
    ctx = contexts.Ctx(req)
    return templates.render(ctx, '/index.mako', repos = conf['repos'])


@wsgihelpers.wsgify
def view_commit(req):
    ctx = contexts.Ctx(req)

    site =  req.params.get('site') or None
    repo_name =  req.params.get('repo_name') or None
    commit_sha1 = req.params.get('commit_sha1', None)

    if repo_name is None or site is None:
        message = "Les données reçu sont incomplete"
        return templates.render(ctx, '/erreur.mako', message = message)

    repo_path = conf['repos'][site][repo_name]
    try:
        repo = Repo(repo_path)
    except NotGitRepository:
        raise wsgihelpers.bad_request(ctx, message = "%s n'est pas un repository GIT" % repo_path)
    except KeyError:
        raise wsgihelpers.bad_request(ctx, message = "%s ne contient pas de références 'HEAD'" % repo_path)
    except:
        raise wsgihelpers.bad_request(ctx, message = "%s n'est pas un repository Git" % repo_path)

    # get the Tree of the Commit and his parent
    commit = repo[commit_sha1]
    commit_date = commit.commit_time
    parents = commit.parents or None
    if parents is None or len(parents) == 0:
        # Initial Commit
        modif_list = []
        for filename, blob in parse_repo(repo_path, commit_sha1):
            modif_list.append((filename, "create"))
        return templates.render_def(ctx, 'commit.mako', 'view_commit', repo_path = repo_path, repo_name = repo_name,
                                    parent_commit_blobs = [], commit_blobs = [], modif_list = modif_list, site = site)
    parent_commit_sha1 = parents[0]

    # Get the list of blob in commit and his parent
    commit_blobs = parse_repo(repo_path, commit_sha1)
    parent_commit_blobs = parse_repo(repo_path, parent_commit_sha1)

    # File modified
    commit_filenames = list()
    parent_commit_filenames = list()
    modif_list = list()
    for blob in commit_blobs:
        commit_filenames.append(blob[0])
    for blob in parent_commit_blobs:
        parent_commit_filenames.append(blob[0])

    # list file wich has been created or deleted
    for filename in commit_filenames:
        if filename not in parent_commit_filenames:
            modif_list.append((filename, "delete"))
    for filename in parent_commit_filenames:
        if filename not in commit_filenames:
            modif_list.append((filename, "create"))

    return templates.render_def(ctx, 'commit.mako', 'view_commit', commit_sha1 = commit_sha1, commit_date = commit_date,
                                repo_path = repo_path, repo_name = repo_name, parent_commit_blobs = parent_commit_blobs,
                                commit_blobs = commit_blobs, modif_list = modif_list, site = site)
