# -*- coding: utf-8 -*-


# MeninGIT -- Web-based wrapper of git
# By: Romain SOUFFLET <rsoufflet@easter-eggs.com>
#
# Copyright (C) 2011 Easter-eggs
# http://wiki.infos-pratiques.org/wiki/MeninGIT
#
# This file is part of meninGIT.
#
# MeninGIT is free software; you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# MeninGIT is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


"""Git Manipulation Class"""

import os

from dulwich import objects
from dulwich.errors import NotGitRepository
from dulwich.repo import Repo, Tree
from dulwich.objects import Blob


TZ = objects.parse_timezone('+100')


def create_tree(repo, path = ''):
    newTree = Tree()
    object_store = repo.object_store
    if len(path) > 0 and path[-1] != '/':
        path = path + "/"

    for entry in os.listdir(repo.path + "/" + path):
        if entry.rfind(".git") == -1:
            gitFileName = repo.path + "/" + path + entry

            if os.path.islink(gitFileName):
                file_content = os.readlink(gitFileName)
                newObject = Blob.from_string(file_content)
                mode = os.lstat(gitFileName).st_mode
                mode = mode - (mode & int("000111111111",2))

            elif os.path.isfile(gitFileName):
                gitFile = open(gitFileName)
                newObject = Blob.from_string(gitFile.read())
                mode = os.stat(gitFileName).st_mode

            elif os.path.isdir(gitFileName):
                newObject = create_tree(repo, path + entry)
                mode = os.stat(gitFileName).st_mode

            newTree.add(mode, entry, newObject.id )
            object_store.add_object(newObject)
    object_store.add_object(newTree)

    return newTree


def has_changed(repo_path):
    try:
        repo = Repo(repo_path)
    except NotGitRepository:
        return False

    # get the Tree of the Commit and his parent
    commit = repo[repo.head()]
    tree_sha1 = commit.tree

    repo_index = repo.open_index()
    changes = list(repo_index.changes_from_tree(repo.object_store, tree_sha1))

    if len(changes) == 0:
        return False
    else:
        return True


def object_store_add_tree(repo, object_store, tree):
    for entry in tree.entries():
        obj = repo.get_object(entry[2])
        if isinstance(obj, Tree):
            object_store = object_store_add_tree(repo, object_store, obj)
    object_store.add_object(tree)
    return object_store


def parse_repo(path, commit_sha1 = None):
    repo = Repo(path)
    try:
        tree_sha1 = repo[commit_sha1 if commit_sha1 is not None else repo.head()].tree
        tree = repo[tree_sha1]
    except:
        return []
    return parse_tree(repo, tree, '')


def parse_tree(repo, tree, path, files = None):
    if files is None:
        files = []
    for entry in tree.entries():
        obj = repo[entry[2]]
        if isinstance(obj, Blob):
            files.append((os.path.join(path, entry[1]), obj))
        elif isinstance(obj, Tree):
            files = parse_tree(repo, obj, os.path.join(path, entry[1]), files)
    return files


def parse_tree_change(repo, path, tree, directory = ''):
    for entry in tree.entries():
        obj = repo[entry[2]]
        if isinstance(obj, Blob):
            if directory != '':
                filename = path + "/" + directory + "/" + entry[1]
            else:
                filename = path + "/" + entry[1]

            if os.path.isfile(filename):
                file = open(filename)
                new_blob = Blob.from_string(file.read())
                if new_blob.id != obj.id:
                    tree.add(0100644, entry[1], new_blob.id)
            else:
                del tree[entry[1]]
        elif isinstance(obj, Tree):
            obj = parse_tree_change(repo, path, obj, directory + entry[1])
        else:
            assert(isinstance(obj, Blob))
    return tree


def print_tree_content(repo, tree, files = None):
    if files is None:
        files = []
    for entry in tree.entries():
        try:
            obj = repo[entry[2]]
        except:
            obj = ''

        if isinstance(obj, Blob):
            files.append(obj.id)
        elif isinstance(obj, Tree):
            files = print_tree_content(repo, obj, files)
    return files

