# -*- coding: utf-8 -*-


# WSGI Helpers -- from Python Toolbox
# By: Emmanuel Raviart <eraviart@easter-eggs.com>
#
# Copyright (C) 2009, 2011 Easter-eggs
#
# This file is part of MeninGIT.
#
# MeninGIT is free software; you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# MeninGIT is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


"""Decorators to wrap functions to make them WSGI applications.

The main decorator :class:`wsgify` turns a function into a WSGI application.
"""


import webob
import webob.dec
import webob.exc
from .. import templates


__all__ = ['error', 'redirect', 'wsgify']


errors_explanation = {
    400: u"La requête est erronée.",
    401: u"L'accès est réservé aux personnes autorisées.",
    403: u"L'accès est interdit.",
    404: u"La page demandée n'a pas été trouvée.",
    }
errors_message = {
    401: u"Vous devez vous identifier pour accéder à cette page.",
    }
errors_title = {
    400: u"Accès impossible",
    401: u"Accès refusé",
    403: u"Accès refusé",
    404: u"Accès impossible",
    }
Request = webob.Request
wsgify = webob.dec.wsgify


def bad_request(ctx, comment = None, explanation = None, headers = None, message = None, title = None):
    return error(ctx, 400, comment = comment, explanation = explanation, headers = headers, message = message,
        title = title)


def error(ctx, code, comment = None, explanation = None, headers = None, message = None, title = None):
    response = webob.exc.status_map[code](headers = headers)
    if code != 204: # No content
        if explanation is None:
            explanation = errors_explanation.get(code)
            explanation = explanation if explanation is not None else response.explanation
        if message is None:
            message = errors_message.get(code)
            if message is not None:
                message = message
        if title is None:
            title = errors_title.get(code)
            title = title if title is not None else response.status
        response.body = templates.render(ctx, '/http-error.mako',
            comment = comment,
            explanation = explanation,
            message = message,
            response = response,
            title = title,
            ).encode('utf-8')
    return response


def forbidden(ctx, comment = None, explanation = None, headers = None, message = None, title = None):
    return error(ctx, 403, comment = comment, explanation = explanation, headers = headers, message = message,
        title = title)


def method_not_allowed(ctx, comment = None, explanation = None, headers = None, message = None, title = None):
    return error(ctx, 405, comment = comment, explanation = explanation, headers = headers, message = message,
        title = title)


def no_content(ctx, headers = None):
    return error(ctx, 204, headers = headers)


def not_found(ctx, comment = None, explanation = None, headers = None, message = None, title = None):
    return error(ctx, 404, comment = comment, explanation = explanation, headers = headers, message = message,
        title = title)


def redirect(ctx, code = 302, location = None, comment = None, explanation = None, headers = None, message = None,
        title = None):
    assert location is not None
    location_str = location.encode('utf-8') if isinstance(location, unicode) else location
    response = webob.exc.status_map[code](headers = headers, location = location_str)
    if explanation is None:
        explanation = "You'll be redirected to page <a href=\"%s\">{1}</a>." % location
    if message is None:
        message = errors_message.get(code)
        if message is not None:
            message = message
    if title is None:
        title = "Redirection in progress..."
    response.body = templates.render(ctx, '/http-error.mako',
        comment = comment,
        explanation = explanation,
        message = message,
        response = response,
        title = title,
        ).encode('utf-8')
    return response


def unauthorized(ctx, comment = None, explanation = None, headers = None, message = None, title = None):
    return error(ctx, 401, comment = comment, explanation = explanation, headers = headers, message = message,
        title = title)


